@@ -1,5 +1,5 @@
 package Git::Raw::Blob;
-$Git::Raw::Blob::VERSION = '0.41';
+$Git::Raw::Blob::VERSION = '0.42';
 use strict;
 use warnings;
 
@@ -11,7 +11,7 @@ Git::Raw::Blob - Git blob class
 
 =head1 VERSION
 
-version 0.41
+version 0.42
 
 =head1 DESCRIPTION
 
@@ -29,7 +29,12 @@ Create a new blob from the given buffer.
 =head2 lookup( $repo, $id )
 
 Retrieve the blob corresponding to C<$id>. This function is pretty much the
-same as C<$repo-E<gt>lookup($id)> except that it only returns blobs.
+same as C<$repo-E<gt>lookup($id)> except that it only returns blobs. If the blob
+doesn't exist, this function wil return C<undef>.
+
+=head2 owner( )
+
+Retrieve the L<Git::Raw::Repository> owning the blob.
 
 =head2 content( )
 
